@@ -66,6 +66,8 @@ struct GSListenerPrivate
         guint           active : 1;
         guint           throttle_enabled : 1;
         GHashTable     *inhibitors;
+
+        time_t          idle_start;
 };
 
 enum {
@@ -163,7 +165,7 @@ gs_listener_send_signal_throttle_enabled_changed (GSListener *listener)
         dbus_message_unref (message);
 }
 
-#if 0
+#ifdef DEBUG_INHIBITORS
 static void
 list_inhibitors (gpointer key,
                  gpointer value,
@@ -182,7 +184,9 @@ listener_check_activation (GSListener *listener)
         if (listener->priv->inhibitors)
                 n_inhibitors = g_hash_table_size (listener->priv->inhibitors);
 
-        /*g_hash_table_foreach (listener->priv->inhibitors, list_inhibitors, NULL);*/
+#ifdef DEBUG_INHIBITORS
+        g_hash_table_foreach (listener->priv->inhibitors, list_inhibitors, NULL);
+#endif
 
         if (listener->priv->idle
             && n_inhibitors == 0) {
@@ -208,21 +212,41 @@ gs_listener_set_active (GSListener *listener,
                         gs_listener_set_throttle_enabled (listener, FALSE);
                         /* if we are deactivating then reset the idle */
                         listener->priv->idle = active;
+                        listener->priv->idle_start = 0;
                 }
         }
 }
 
-void
+gboolean
 gs_listener_set_idle (GSListener *listener,
                       gboolean    idle)
 {
-        g_return_if_fail (GS_IS_LISTENER (listener));
+        g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
 
-        if (listener->priv->idle != idle) {
-                listener->priv->idle = idle;
+        if (listener->priv->idle == idle)
+                return FALSE;
 
-                listener_check_activation (listener);
+        if (idle) {
+                guint n_inhibitors = 0;
+
+                /* if we aren't inhibited then set idle */
+                if (listener->priv->inhibitors)
+                        n_inhibitors = g_hash_table_size (listener->priv->inhibitors);
+
+                if (n_inhibitors != 0) {
+                        return FALSE;
+                }
+
+                listener->priv->idle_start = time (NULL);
+        } else {
+                listener->priv->idle_start = 0;
         }
+
+        listener->priv->idle = idle;
+
+        listener_check_activation (listener);
+
+        return TRUE;
 }
 
 void
@@ -539,6 +563,48 @@ listener_get_property (GSListener     *listener,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+listener_get_idle_time (GSListener     *listener,
+                        DBusConnection *connection,
+                        DBusMessage    *message)
+{
+        const char     *path;
+        DBusMessageIter iter;
+        DBusMessage    *reply;
+        dbus_uint32_t    secs;
+
+        path = dbus_message_get_path (message);
+
+        reply = dbus_message_new_method_return (message);
+
+        dbus_message_iter_init_append (reply, &iter);
+
+        if (reply == NULL)
+                g_error ("No memory");
+
+        if (listener->priv->idle) {
+                time_t now = time (NULL);
+
+                if (now < listener->priv->idle_start) {
+                        /* should't happen */
+                        g_warning ("Idle start time is in the future");
+                        secs = 0;
+                } else {
+                        secs = listener->priv->idle_start - now;
+                }
+        } else {
+                secs = 0;
+        }
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &secs);
+
+        if (! dbus_connection_send (connection, reply, NULL))
+                g_error ("No memory");
+
+        dbus_message_unref (reply);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 listener_dbus_filter_handle_methods (DBusConnection *connection,
                                      DBusMessage    *message, 
@@ -577,6 +643,9 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getIdle")) {
                 return listener_get_property (listener, connection, message, PROP_IDLE);
         }
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getIdleTime")) {
+                return listener_get_idle_time (listener, connection, message);
+        }
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "setThrottleEnabled")) {
                 return listener_set_property (listener, connection, message, PROP_THROTTLE_ENABLED);
         }
